"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIEditorContribution = void 0;
const tslib_1 = require("tslib");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-core-browser/lib/menu/next/renderer/ctxmenu/browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const menu_service_1 = require("@opensumi/ide-design/lib/browser/override/menu.service");
const browser_2 = require("@opensumi/ide-editor/lib/browser");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const telemetry_service_1 = require("@opensumi/ide-monaco/lib/browser/telemetry.service");
const ai_native_service_1 = require("./ai-native.service");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const service_1 = require("./languages/service");
const rename_service_1 = require("./rename/rename.service");
const types_1 = require("./types");
const inline_chat_feature_registry_1 = require("./widget/inline-chat/inline-chat.feature.registry");
const inline_chat_service_1 = require("./widget/inline-chat/inline-chat.service");
const inline_content_widget_1 = require("./widget/inline-chat/inline-content-widget");
const inline_diff_widget_1 = require("./widget/inline-diff/inline-diff-widget");
let AIEditorContribution = class AIEditorContribution extends ide_core_common_1.Disposable {
    constructor() {
        super();
        this.aiInlineChatDisposed = new ide_core_common_1.Disposable();
        this.aiInlineChatOperationDisposed = new ide_core_common_1.Disposable();
        this.inlineChatInUsing = false;
        this.logger = this.loggerManagerClient.getLogger(ide_core_common_1.SupportLogNamespace.Browser);
    }
    disposeAllWidget() {
        [
            this.aiDiffWidget,
            this.aiInlineContentWidget,
            this.aiInlineChatDisposed,
            this.aiInlineChatOperationDisposed,
        ].forEach((widget) => {
            widget === null || widget === void 0 ? void 0 : widget.dispose();
        });
        this.aiNativeService.cancelToken();
        this.inlineChatInUsing = false;
    }
    contribute(editor) {
        if (!editor) {
            return this;
        }
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return this;
        }
        this.contributeInlineCompletionFeature(editor);
        this.contributeInlineChatFeature(editor);
        this.registerLanguageFeatures(editor);
        this.disposables.push(monacoEditor.onDidScrollChange(() => {
            /**
             * 其他的 ctxmenu 服务注册的菜单在 onHide 函数里会有其他逻辑处理，例如在 editor.context.ts 会在 hide 的时候 focus 编辑器，影响使用
             */
            this.ctxMenuRenderer.onHide = undefined;
            this.ctxMenuRenderer.hide(true);
        }));
        return this;
    }
    contributeInlineCompletionFeature(editor) {
        const { monacoEditor } = editor;
        // 判断用户是否选择了一块区域或者移动光标 取消掉请补全求
        const selectionChange = () => {
            this.aiCompletionsService.hideStatusBarItem();
            const selection = monacoEditor.getSelection();
            if (!selection) {
                return;
            }
            // 判断是否选中区域
            if (selection.startLineNumber !== selection.endLineNumber || selection.startColumn !== selection.endColumn) {
                this.aiInlineCompletionsProvider.cancelRequest();
            }
            requestAnimationFrame(() => {
                this.aiCompletionsService.setVisibleCompletion(false);
            });
        };
        const debouncedSelectionChange = (0, debounce_1.default)(selectionChange, 50, {
            maxWait: 200,
            leading: true,
            trailing: true,
        });
        this.disposables.push(this.eventBus.on(browser_2.EditorSelectionChangeEvent, (e) => {
            if (e.payload.source === 'mouse') {
                debouncedSelectionChange();
            }
            else {
                debouncedSelectionChange.cancel();
                selectionChange();
            }
        }), monacoEditor.onDidChangeModelContent((e) => {
            const changes = e.changes;
            for (const change of changes) {
                if (change.text === '') {
                    this.aiInlineCompletionsProvider.isDelEvent = true;
                    this.aiInlineCompletionsProvider.cancelRequest();
                }
                else {
                    this.aiInlineCompletionsProvider.isDelEvent = false;
                }
            }
        }), monacoEditor.onWillChangeModel(() => {
            this.aiCompletionsService.hideStatusBarItem();
            this.disposeAllWidget();
        }), monacoEditor.onDidBlurEditorText(() => {
            this.aiCompletionsService.hideStatusBarItem();
            this.aiCompletionsService.setVisibleCompletion(false);
        }));
    }
    contributeInlineChatFeature(editor) {
        const { monacoEditor } = editor;
        this.disposables.push(this.aiNativeService.onInlineChatVisible((value) => {
            if (value) {
                this.showInlineChat(editor);
            }
            else {
                this.disposeAllWidget();
            }
        }), 
        // 通过 code actions 来透出我们 inline chat 的功能
        this.inlineChatFeatureRegistry.onCodeActionRun(({ id, range }) => {
            monacoEditor.setSelection(range);
            this.showInlineChat(editor);
            if (this.aiInlineContentWidget) {
                this.aiInlineContentWidget.clickActionId(id, 'codeAction');
            }
        }));
        let needShowInlineChat = false;
        this.disposables.push(monacoEditor.onMouseDown(() => {
            needShowInlineChat = false;
        }), monacoEditor.onMouseUp((event) => {
            const target = event.target;
            const detail = target.detail;
            if (detail && typeof detail === 'string' && detail === ide_core_common_1.AIInlineChatContentWidgetId) {
                needShowInlineChat = false;
            }
            else {
                needShowInlineChat = true;
            }
        }));
        let prefInlineChatAutoVisible = this.preferenceService.getValid(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, true);
        this.disposables.push(this.preferenceService.onSpecificPreferenceChange(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, ({ newValue }) => {
            prefInlineChatAutoVisible = newValue;
        }));
        this.disposables.push(ide_core_common_1.Event.debounce(ide_core_common_1.Event.any(monacoEditor.onDidChangeCursorSelection, monacoEditor.onMouseUp), (_, e) => e, 100)(() => {
            if (!prefInlineChatAutoVisible || !needShowInlineChat) {
                return;
            }
            if (this.aiInlineChatService.status !== 0 /* EInlineChatStatus.READY */ &&
                this.aiInlineChatService.status !== 3 /* EInlineChatStatus.ERROR */) {
                return;
            }
            this.showInlineChat(editor);
        }));
    }
    async showInlineChat(editor) {
        if (!this.aiNativeConfigService.capabilities.supportsInlineChat) {
            return;
        }
        if (this.inlineChatInUsing) {
            return;
        }
        this.inlineChatInUsing = true;
        this.disposeAllWidget();
        const { monacoEditor } = editor;
        const selection = monacoEditor.getSelection();
        if (!selection || selection.isEmpty()) {
            this.disposeAllWidget();
            return;
        }
        this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */));
        this.aiInlineContentWidget = this.injector.get(inline_content_widget_1.AIInlineContentWidget, [monacoEditor]);
        this.aiInlineContentWidget.show({
            selection,
        });
        this.aiInlineChatDisposed.addDispose(this.aiInlineContentWidget.onActionClick((action) => {
            this.runInlineChatAction(action, monacoEditor);
        }));
    }
    async runInlineChatAction({ actionId: id, source, }, monacoEditor) {
        const handler = this.inlineChatFeatureRegistry.getEditorHandler(id);
        const action = this.inlineChatFeatureRegistry.getAction(id);
        if (!handler || !action) {
            return;
        }
        const selection = monacoEditor.getSelection();
        if (!selection) {
            this.logger.error('No selection found, aborting inline chat action.');
            return;
        }
        const { execute, providerDiffPreviewStrategy } = handler;
        if (execute) {
            await execute(monacoEditor);
            this.disposeAllWidget();
        }
        if (providerDiffPreviewStrategy) {
            const crossSelection = selection
                .setStartPosition(selection.startLineNumber, 1)
                .setEndPosition(selection.endLineNumber, Number.MAX_SAFE_INTEGER);
            const relationId = this.aiReporter.start(action.name, {
                message: action.name,
                type: ide_core_common_1.AISerivceType.InlineChat,
                source,
                runByCodeAction: source === 'codeAction',
            });
            const result = await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection, relationId, false);
            this.aiInlineChatDisposed.addDispose([
                this.aiInlineChatService.onDiscard(() => {
                    this.aiReporter.end(relationId, { message: result.message, success: true, isDrop: true });
                    this.disposeAllWidget();
                }),
                this.aiInlineChatService.onRegenerate(async () => {
                    await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection, relationId, true);
                }),
            ]);
        }
    }
    async handleDiffPreviewStrategy(monacoEditor, strategy, crossSelection, relationId, isRetry) {
        const model = monacoEditor.getModel();
        this.resetDiffEnvironment();
        const crossCode = model.getValueInRange(crossSelection);
        this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.launchChatStatus(1 /* EInlineChatStatus.THINKING */));
        const startTime = Date.now();
        const response = await strategy(monacoEditor, this.aiNativeService.cancelIndicator.token);
        if (this.aiInlineChatDisposed.disposed || ide_core_common_1.CancelResponse.is(response)) {
            this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */));
            this.aiReporter.end(relationId, {
                message: response.message,
                success: true,
                replytime: Date.now() - startTime,
                isStop: true,
                isRetry,
            });
            return response;
        }
        if (ide_core_common_1.ErrorResponse.is(response)) {
            this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.launchChatStatus(3 /* EInlineChatStatus.ERROR */));
            this.aiReporter.end(relationId, {
                message: response.message,
                success: false,
                replytime: Date.now() - startTime,
                isRetry,
            });
            return response;
        }
        this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.launchChatStatus(2 /* EInlineChatStatus.DONE */));
        this.aiReporter.end(relationId, {
            message: response.message,
            success: true,
            replytime: Date.now() - startTime,
            isRetry,
        });
        let answer = this.extractAnswerFromResponse(response);
        answer = this.formatAnswer(answer, crossCode);
        this.visibleDiffWidget(monacoEditor, crossSelection, answer);
        this.aiInlineChatOperationDisposed.addDispose([
            this.aiInlineChatService.onAccept(() => {
                var _a;
                this.aiReporter.end(relationId, { message: 'accept', success: true, isReceive: true });
                (_a = monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.pushEditOperations(null, [{ range: crossSelection, text: answer }], () => null);
                (0, ide_core_common_1.runWhenIdle)(() => {
                    this.disposeAllWidget();
                });
            }),
            this.aiInlineChatService.onThumbs((isLike) => {
                this.aiReporter.end(relationId, { isLike });
            }),
            this.aiDiffWidget.onMaxLincCount((count) => {
                requestAnimationFrame(() => {
                    if (crossSelection.endLineNumber === model.getLineCount()) {
                        const lineHeight = monacoEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
                        this.aiInlineContentWidget.offsetTop(lineHeight * count + 12);
                    }
                });
            }),
        ]);
        return response;
    }
    resetDiffEnvironment() {
        var _a;
        (_a = this.aiDiffWidget) === null || _a === void 0 ? void 0 : _a.dispose();
        this.aiInlineChatOperationDisposed.dispose();
    }
    extractAnswerFromResponse(response) {
        const regex = /```\w*([\s\S]+?)\s*```/;
        const match = regex.exec(response.message);
        return match ? match[1].trim() : response.message.trim();
    }
    formatAnswer(answer, crossCode) {
        const leadingWhitespaceMatch = crossCode.match(/^\s*/);
        const indent = leadingWhitespaceMatch ? leadingWhitespaceMatch[0] : '  ';
        return answer
            .split('\n')
            .map((line, index) => (index === 0 ? line : `${indent}${line}`))
            .join('\n');
    }
    visibleDiffWidget(monacoEditor, crossSelection, answer) {
        monacoEditor.setHiddenAreas([crossSelection], inline_diff_widget_1.AIDiffWidget._hideId);
        this.aiDiffWidget = this.injector.get(inline_diff_widget_1.AIDiffWidget, [monacoEditor, crossSelection, answer]);
        this.aiDiffWidget.create();
        this.aiDiffWidget.showByLine(crossSelection.startLineNumber - 1, crossSelection.endLineNumber - crossSelection.startLineNumber + 2);
        this.updateInlineContentWidgetPosition(crossSelection);
    }
    updateInlineContentWidgetPosition(crossSelection) {
        var _a, _b;
        (_a = this.aiInlineContentWidget) === null || _a === void 0 ? void 0 : _a.setOptions({
            position: {
                lineNumber: crossSelection.endLineNumber + 1,
                column: 1,
            },
        });
        (_b = this.aiInlineContentWidget) === null || _b === void 0 ? void 0 : _b.layoutContentWidget();
    }
    async registerLanguageFeatures(editor) {
        const { monacoEditor } = editor;
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async () => {
            if (this.modelSessionDisposable) {
                this.modelSessionDisposable.dispose();
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            this.modelSessionDisposable = new ide_core_common_1.Disposable();
            const languageId = model.getLanguageId();
            if (this.aiNativeConfigService.capabilities.supportsInlineCompletion) {
                this.contributions.getContributions().forEach((contribution) => {
                    if (contribution.middleware) {
                        this.latestMiddlewareCollector = contribution.middleware;
                    }
                });
                this.aiInlineCompletionsProvider.registerEditor(editor);
                this.modelSessionDisposable.addDispose({
                    dispose: () => {
                        this.aiInlineCompletionsProvider.dispose();
                    },
                });
                this.modelSessionDisposable.addDispose(monaco_api_1.monaco.languages.registerInlineCompletionsProvider(languageId, {
                    provideInlineCompletions: async (model, position, context, token) => {
                        var _a, _b, _c, _d;
                        if ((_b = (_a = this.latestMiddlewareCollector) === null || _a === void 0 ? void 0 : _a.language) === null || _b === void 0 ? void 0 : _b.provideInlineCompletions) {
                            this.aiCompletionsService.setMiddlewareComplete((_d = (_c = this.latestMiddlewareCollector) === null || _c === void 0 ? void 0 : _c.language) === null || _d === void 0 ? void 0 : _d.provideInlineCompletions);
                        }
                        const list = await this.aiInlineCompletionsProvider.provideInlineCompletionItems(model, position, context, token);
                        this.logger.log('provideInlineCompletions: ', list.items.map((data) => data.insertText));
                        return list;
                    },
                    freeInlineCompletions() { },
                    handleItemDidShow: (completions) => {
                        if (completions.items.length > 0) {
                            this.aiCompletionsService.setVisibleCompletion(true);
                        }
                    },
                }));
            }
            if (this.aiNativeConfigService.capabilities.supportsRenameSuggestions) {
                this.modelSessionDisposable.addDispose(this.contributeRenameFeature(languageId));
            }
            if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
                this.modelSessionDisposable.addDispose(this.contributeCodeActionFeature(languageId, editor));
            }
        }));
    }
    contributeRenameFeature(languageId) {
        const disposable = new ide_core_common_1.Disposable();
        const provider = async (model, range, token) => {
            this.lastModelRequestRenameSessionId = undefined;
            const startTime = +new Date();
            const relationId = this.aiReporter.start('rename', {
                message: 'start',
                type: ide_core_common_1.AISerivceType.Rename,
                modelRequestStartTime: startTime,
            });
            this.lastModelRequestRenameSessionId = relationId;
            const toDispose = token.onCancellationRequested(() => {
                const endTime = +new Date();
                this.aiReporter.end(relationId, {
                    message: 'cancel',
                    success: false,
                    isCancel: true,
                    modelRequestStartTime: startTime,
                    modelRequestEndTime: endTime,
                });
                this.lastModelRequestRenameSessionId = undefined;
            });
            try {
                const result = await this.renameSuggestionService.provideRenameSuggestions(model, range, token);
                toDispose.dispose();
                this.lastModelRequestRenameEndTime = +new Date();
                return result;
            }
            catch (error) {
                const endTime = +new Date();
                this.aiReporter.end(relationId, {
                    message: 'error:' + (0, ide_core_common_1.getErrorMessage)(error),
                    success: false,
                    modelRequestStartTime: startTime,
                    modelRequestEndTime: endTime,
                });
                throw error;
            }
        };
        disposable.addDispose([
            monaco_api_1.monaco.languages.registerNewSymbolNameProvider(languageId, {
                provideNewSymbolNames: provider,
            }),
            this.monacoTelemetryService.onEventLog('renameInvokedEvent', (event) => {
                if (this.lastModelRequestRenameSessionId) {
                    this.aiReporter.end(this.lastModelRequestRenameSessionId, Object.assign({ message: 'done', success: true, modelRequestEndTime: this.lastModelRequestRenameEndTime }, event));
                }
            }),
        ]);
        return disposable;
    }
    contributeCodeActionFeature(languageId, editor) {
        const disposable = new ide_core_common_1.Disposable();
        let prefInlineChatActionEnabled = this.preferenceService.getValid(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_CODE_ACTION_ENABLED, true);
        if (!prefInlineChatActionEnabled) {
            return disposable;
        }
        const { monacoEditor } = editor;
        const { languageParserService, inlineChatFeatureRegistry } = this;
        let codeActionDispose;
        disposable.addDispose(this.preferenceService.onSpecificPreferenceChange(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_CODE_ACTION_ENABLED, ({ newValue }) => {
            prefInlineChatActionEnabled = newValue;
            if (newValue) {
                register();
            }
            else {
                if (codeActionDispose) {
                    codeActionDispose.dispose();
                    codeActionDispose = undefined;
                }
            }
        }));
        register();
        return disposable;
        function register() {
            if (codeActionDispose) {
                codeActionDispose.dispose();
                codeActionDispose = undefined;
            }
            codeActionDispose = monaco_api_1.monaco.languages.registerCodeActionProvider(languageId, {
                provideCodeActions: async (model) => {
                    if (!prefInlineChatActionEnabled) {
                        return;
                    }
                    const parser = languageParserService.createParser(languageId);
                    if (!parser) {
                        return;
                    }
                    const actions = inlineChatFeatureRegistry.getCodeActions();
                    if (!actions || actions.length === 0) {
                        return;
                    }
                    const cursorPosition = monacoEditor.getPosition();
                    if (!cursorPosition) {
                        return;
                    }
                    function constructCodeActions(info) {
                        return {
                            actions: actions.map((v) => {
                                const command = {};
                                if (v.command) {
                                    command.id = v.command.id;
                                    command.arguments = [info.range];
                                }
                                let title = v.title;
                                switch (info.infoCategory) {
                                    case 'function': {
                                        title = title + ` for Function: ${info.name}`;
                                    }
                                }
                                return Object.assign(Object.assign({}, v), { title, ranges: [info.range], command });
                            }),
                            dispose() { },
                        };
                    }
                    const info = await parser.provideCodeBlockInfo(model, cursorPosition);
                    if (info) {
                        return constructCodeActions(info);
                    }
                    // check current line is empty
                    const currentLineLength = model.getLineLength(cursorPosition.lineNumber);
                    if (currentLineLength !== 0) {
                        return;
                    }
                    // 获取视窗范围内的代码块
                    const ranges = monacoEditor.getVisibleRanges();
                    if (ranges.length === 0) {
                        return;
                    }
                    // 查找从当前行至视窗最后一行的代码块中是否包含函数
                    const newRange = new monaco.Range(cursorPosition.lineNumber, 0, ranges[0].endLineNumber + 1, 0);
                    const rangeInfo = await parser.provideCodeBlockInfoInRange(model, newRange);
                    if (rangeInfo) {
                        return constructCodeActions(rangeInfo);
                    }
                },
            });
            disposable.addDispose(codeActionDispose);
        }
    }
    dispose() {
        super.dispose();
        if (this.modelSessionDisposable) {
            this.modelSessionDisposable.dispose();
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIEditorContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], AIEditorContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AIInlineChatService)
], AIEditorContribution.prototype, "aiInlineChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_native_service_1.AINativeService),
    tslib_1.__metadata("design:type", ai_native_service_1.AINativeService)
], AIEditorContribution.prototype, "aiNativeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ILoggerManagerClient),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "loggerManagerClient", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.IBrowserCtxMenu),
    tslib_1.__metadata("design:type", menu_service_1.DesignBrowserCtxMenuService)
], AIEditorContribution.prototype, "ctxMenuRenderer", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AIEditorContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "aiReporter", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AINativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AIInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AIInlineCompletionsProvider)
], AIEditorContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(rename_service_1.RenameSuggestionsService),
    tslib_1.__metadata("design:type", rename_service_1.RenameSuggestionsService)
], AIEditorContribution.prototype, "renameSuggestionService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AIEditorContribution.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IEventBus),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "eventBus", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(service_1.LanguageParserService),
    tslib_1.__metadata("design:type", service_1.LanguageParserService)
], AIEditorContribution.prototype, "languageParserService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", telemetry_service_1.MonacoTelemetryService)
], AIEditorContribution.prototype, "monacoTelemetryService", void 0);
AIEditorContribution = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], AIEditorContribution);
exports.AIEditorContribution = AIEditorContribution;
//# sourceMappingURL=ai-editor.contribution.js.map