"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AITerminalService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_terminal_next_1 = require("@opensumi/ide-terminal-next");
const inline_chat_feature_registry_1 = require("../widget/inline-chat/inline-chat.feature.registry");
const terminal_decoration_1 = require("./decoration/terminal-decoration");
const matcher_1 = require("./matcher");
const ansi_parser_1 = require("./utils/ansi-parser");
let AITerminalService = class AITerminalService extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.outputRecordMap = new Map();
        this.inputRecordMap = new Map();
        this.clientCurrentMarker = new Map();
        this.inputCache = '';
    }
    active() {
        this.disposables.push(this.terminalController.onDidOpenTerminal(({ id }) => this.listenTerminalEvent(id)));
    }
    listenTerminalEvent(clientId) {
        const client = this.terminalController.clients.get(clientId);
        if (client) {
            this.disposables.push(client.onOutput(({ id, data }) => this.handleClientOutput(id, data)), client.onInput(({ id, data }) => this.handleClientInput(id, data)));
            this.resetState(clientId);
        }
    }
    resetState(clientId) {
        this.inputCache = '';
        this.inputRecordMap.set(clientId, []);
        this.outputRecordMap.set(clientId, []);
        this.clientCurrentMarker.set(clientId, undefined);
        this.currentActions = null;
    }
    /**
     * 按行解析输出文本
     */
    handleClientOutput(clientId, data) {
        if (!this.isTyping) {
            const client = this.terminalController.clients.get(clientId);
            const viewportCol = client.term.cols;
            const dataLine = data.toString().split('\r\n');
            let dataLineIndex = 0;
            dataLine.forEach((line, index) => {
                const { parts, currentStyle } = (0, ansi_parser_1.ansiParser)(line, this.lastStyle);
                this.lastStyle = currentStyle;
                this.matchOutput(clientId, parts, dataLineIndex || index);
                // 一次渲染多个错误时，需要计算错误所在的行数
                // 因为终端窗口大小的关系，文字可能会换行，需要记录下实际的行数
                const lineText = parts.map((p) => p.content).join('');
                const dataRenderLineNum = Math.ceil(lineText.length / viewportCol);
                dataLineIndex += Math.max(1, dataRenderLineNum);
            });
        }
    }
    getMatcherRules() {
        const allActions = this.inlineChatFeatureRegistry.getTerminalActions();
        const matcher = [];
        allActions.forEach((action) => {
            const handler = this.inlineChatFeatureRegistry.getTerminalHandler(action.id);
            if (Array.isArray(handler === null || handler === void 0 ? void 0 : handler.triggerRules)) {
                handler.triggerRules.forEach((rule) => {
                    if (rule instanceof matcher_1.BaseTerminalDetectionLineMatcher) {
                        matcher.push({
                            matcher: rule,
                            action,
                        });
                    }
                    else if (typeof rule === 'function' && Object.getPrototypeOf(rule) === matcher_1.BaseTerminalDetectionLineMatcher) {
                        matcher.push({
                            matcher: new rule(),
                            action,
                        });
                    }
                });
            }
        });
        return matcher;
    }
    /**
     * 匹配错误日志， 两种情况
     * 1. 单行错误：匹配之后直接上报
     * 2. 多行错误：缓存当前匹配的 matcher，下一行继续使用
     *    2.1 如果匹配到，缓存该行内容
     *    2.2 如果没匹配到，直接上报前面多行的内容
     */
    matchOutput(clientId, styleList, dataLineIndex) {
        var _a, _b;
        if (this.currentActions) {
            if (this.currentActions.matcher.match(styleList)) {
                (_a = this.outputRecordMap
                    .get(clientId)) === null || _a === void 0 ? void 0 : _a.push({ type: this.currentActions.matcher.type, text: styleList.map((s) => s.content).join('') });
            }
            else {
                this.matchedEnd(clientId, this.currentActions);
            }
        }
        else {
            const matcherList = this.getMatcherRules();
            const findMatcher = matcherList.find((m) => m.matcher.match(styleList));
            if (findMatcher) {
                this.registerMarker(clientId, dataLineIndex);
                (_b = this.outputRecordMap
                    .get(clientId)) === null || _b === void 0 ? void 0 : _b.push({ type: findMatcher.matcher.type, text: styleList.map((s) => s.content).join('') });
                if (findMatcher.matcher.isMultiLine) {
                    this.currentActions = {
                        action: findMatcher.action,
                        matcher: findMatcher.matcher,
                    };
                }
                else {
                    this.matchedEnd(clientId, findMatcher);
                }
            }
        }
    }
    matchedEnd(clientId, matcher) {
        var _a, _b;
        const input = this.resolveDelControl((_a = this.inputRecordMap
            .get(clientId)) === null || _a === void 0 ? void 0 : _a.map((r) => r.text).join('\n'));
        const output = (_b = this.outputRecordMap
            .get(clientId)) === null || _b === void 0 ? void 0 : _b.map((r) => r.text).join('\n');
        this.addDecoration(clientId, matcher, input, output);
        this.report(matcher.matcher.type, input, output);
        this.resetState(clientId);
    }
    addDecoration(clientId, action, input, output) {
        const client = this.terminalController.clients.get(clientId);
        const terminal = client === null || client === void 0 ? void 0 : client.term;
        const marker = this.clientCurrentMarker.get(clientId);
        if (terminal && output && marker) {
            const lines = output === null || output === void 0 ? void 0 : output.split('\n').length;
            this.terminalDecorations.addZoneDecoration(terminal, marker, lines, {
                operationList: [action.action],
                onClickItem: () => {
                    const handler = this.inlineChatFeatureRegistry.getTerminalHandler(action.action.id);
                    if (handler) {
                        handler.execute(output, input || '', action.matcher);
                    }
                },
            });
        }
    }
    resolveDelControl(text = '') {
        const charArray = text.split('');
        let i = 0;
        while (i < charArray.length) {
            if (charArray[i].charCodeAt(0) === 127) {
                // 如果遇到 DEL 控制字符，删除其前面的字符
                const deleteIndex = Math.max(i - 1, 0);
                charArray.splice(deleteIndex, 1);
                i = deleteIndex;
            }
            else {
                i++;
            }
        }
        return charArray.join('');
    }
    report(type, input, output) {
        // 上报埋点
    }
    registerMarker(clientId, dataLineIndex) {
        const client = this.terminalController.clients.get(clientId);
        if (client) {
            const marker = client.term.registerMarker(dataLineIndex);
            this.clientCurrentMarker.set(clientId, marker);
            this.disposables.push(marker);
        }
    }
    handleClientInput(clientId, data) {
        const inputString = data.toString();
        const recordList = this.inputRecordMap.get(clientId);
        // 如果输入回车，则认为执行命令
        if (/[\r\n]/.test(inputString)) {
            this.isTyping = false;
            recordList === null || recordList === void 0 ? void 0 : recordList.push({ text: this.inputCache });
        }
        else {
            // 缓存输入
            this.inputCache += inputString;
            this.isTyping = true;
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_terminal_next_1.ITerminalController),
    tslib_1.__metadata("design:type", Object)
], AITerminalService.prototype, "terminalController", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(terminal_decoration_1.AITerminalDecorationService),
    tslib_1.__metadata("design:type", terminal_decoration_1.AITerminalDecorationService)
], AITerminalService.prototype, "terminalDecorations", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AITerminalService.prototype, "inlineChatFeatureRegistry", void 0);
AITerminalService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], AITerminalService);
exports.AITerminalService = AITerminalService;
//# sourceMappingURL=ai-terminal.service.js.map