"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageParser = void 0;
const tslib_1 = require("tslib");
const web_tree_sitter_1 = tslib_1.__importDefault(require("web-tree-sitter"));
const di_1 = require("@opensumi/di");
const ide_utils_1 = require("@opensumi/ide-utils");
const common_1 = require("./tree-sitter/common");
const language_facts_1 = require("./tree-sitter/language-facts");
const wasm_manager_1 = require("./tree-sitter/wasm-manager");
let LanguageParser = class LanguageParser {
    constructor(language) {
        this.language = language;
        this.parserLoaded = new ide_utils_1.Deferred();
        this.lruCache = new ide_utils_1.LRUCache(500);
        this.initializeParser();
    }
    ready() {
        return this.parserLoaded.promise;
    }
    async initializeParser() {
        this.parser = await this.wasmModuleManager.initParser();
        // Load grammar
        const grammar = await this.wasmModuleManager.loadLanguage(this.language);
        const languageParser = await web_tree_sitter_1.default.Language.load(new Uint8Array(grammar));
        // Set language
        this.parser.setLanguage(languageParser);
        this.parserLoaded.resolve();
    }
    /**
     * 从给定的位置开始，向上遍历 AST，找到最近的代码块
     * @param node 节点
     * @param position 位置
     * @returns 代码块
     */
    findContainingCodeBlockWithPosition(node, position) {
        if (node.startIndex <= position && node.endIndex >= position) {
            const isBlockIdentifier = this.languageFacts.isCodeBlock(this.language, node.type);
            if (isBlockIdentifier) {
                return node;
            }
        }
        if (node.parent) {
            return this.findContainingCodeBlockWithPosition(node.parent, position);
        }
        return null;
    }
    findFunctionCodeBlock(node, position) {
        if (node.startIndex <= position && node.endIndex >= position) {
            const valid = this.languageFacts.isFunctionCodeBlock(this.language, node.type);
            if (valid) {
                return node;
            }
        }
        if (node.parent) {
            return this.findFunctionCodeBlock(node.parent, position);
        }
        return null;
    }
    async parseAST(model) {
        const key = `${model.id}@${model.getVersionId()}`;
        const cachedNode = this.lruCache.get(key);
        if (cachedNode) {
            return cachedNode;
        }
        await this.parserLoaded.promise;
        const sourceCode = model.getValue();
        const tree = this.parser.parse(sourceCode);
        if (tree) {
            const rootNode = tree.rootNode;
            this.lruCache.set(key, rootNode);
            return rootNode;
        }
    }
    async getSyntaxNodeAsPosition(model, cursor) {
        const rootNode = await this.parseAST(model);
        if (rootNode) {
            const cursorNode = rootNode.namedDescendantForIndex(cursor);
            return cursorNode;
        }
        return null;
    }
    /**
     * 从给定的位置开始，找到最近的没有语法错误的代码块
     */
    async findCodeBlockWithSyntaxError(sourceCode, range) {
        await this.parserLoaded.promise;
        const tree = this.parser.parse(sourceCode);
        if (tree) {
            const rootNode = tree.rootNode;
            const startPosition = {
                row: range.startLineNumber - 1,
                column: range.startColumn,
            };
            const selectedNode = rootNode.namedDescendantForPosition(startPosition);
            let parentNode = selectedNode.parent;
            if (!parentNode) {
                return {
                    range: (0, common_1.toMonacoRange)(selectedNode),
                    type: selectedNode.type,
                    infoCategory: 'other',
                };
            }
            // 检查父节点是否有语法错误，并向上遍历
            while (parentNode) {
                // 检查当前节点是否有错误
                const hasError = parentNode.hasError;
                // 如果有错误，停止遍历，当前节点的父节点即为所求的代码块
                if (hasError) {
                    break;
                }
                // 向上移动到父节点
                parentNode = parentNode.parent;
            }
            if (parentNode) {
                return {
                    range: (0, common_1.toMonacoRange)(parentNode),
                    type: parentNode.type,
                    infoCategory: 'other',
                };
            }
            return {
                range,
                type: selectedNode.type,
                infoCategory: 'other',
            };
        }
        return null;
    }
    async provideCodeBlockInfo(model, position) {
        const cursor = model.getOffsetAt(position);
        const cursorNode = await this.getSyntaxNodeAsPosition(model, cursor);
        if (!cursorNode) {
            return null;
        }
        const functionNode = this.findFunctionCodeBlock(cursorNode, cursor);
        if (functionNode) {
            return this.languageFacts.provideFunctionInfo(this.language, functionNode);
        }
        const selectedNode = this.findContainingCodeBlockWithPosition(cursorNode, cursor);
        if (selectedNode) {
            return {
                infoCategory: 'other',
                range: {
                    startLineNumber: selectedNode.startPosition.row + 1,
                    startColumn: 0,
                    endLineNumber: selectedNode.endPosition.row + 1,
                    endColumn: Infinity,
                },
                type: selectedNode.type,
            };
        }
        return null;
    }
    async provideCodeBlockInfoInRange(model, range) {
        const rootNode = await this.parseAST(model);
        if (rootNode) {
            const startPosition = {
                row: range.startLineNumber - 1,
                column: range.startColumn - 1,
            };
            const endPosition = {
                row: range.endLineNumber,
                column: range.endColumn,
            };
            const types = this.languageFacts.getCodeBlockTypes(this.language);
            if (!types || types.size === 0) {
                return null;
            }
            const nodes = rootNode.descendantsOfType(Array.from(types), startPosition, endPosition);
            if (nodes && nodes.length > 0) {
                const firstNode = nodes[0];
                const range = (0, common_1.toMonacoRange)(firstNode);
                return {
                    infoCategory: 'other',
                    range,
                    type: firstNode.type,
                };
            }
        }
        return null;
    }
    dispose() {
        this.parser.delete();
        this.lruCache.clear();
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(wasm_manager_1.WasmModuleManager),
    tslib_1.__metadata("design:type", wasm_manager_1.WasmModuleManager)
], LanguageParser.prototype, "wasmModuleManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(language_facts_1.TreeSitterLanguageFacts),
    tslib_1.__metadata("design:type", language_facts_1.TreeSitterLanguageFacts)
], LanguageParser.prototype, "languageFacts", void 0);
LanguageParser = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [String])
], LanguageParser);
exports.LanguageParser = LanguageParser;
//# sourceMappingURL=parser.js.map